/*
 In-line Network Cable Tracer (INCT)
 Version: 0.3
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 22/04/2013
 Last Modified: 16/07/2013
 
 Connect 2x INCT to each end of your network cable.
 When the button is held down for 1-2 seconds, the 3V battery with 220 Ohm resistor connects to RX-/+ and TX-/+ lines
 and then LED will turn on.
 
 Change your fuse bits so the ATtint25/45/85 uses the 128KHz internal oscillator:
 avrdude -p ATtiny84 -c usbtiny -U lfuse:w:0xe4:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m 
 
 */

#define F_CPU 128000 // 128 KHz

#define adcPin 0 // PA0
#define ledPin PB0
#define txPlusControlPin PA2
#define txMinusControlPin PA3
#define rxPlusControlPin PA4
#define rxMinusControlPin PA5

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "setup.c"
 
// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//     LED PB0  2|    |13  PA0 ADC
//         PB1  3|    |12  PA1
//   Reset PB3  4|    |11  PA2 TX+
//         PB2  5|    |10  PA3 TX-
//         PA7  6|    |9   PA4 RX+
//         PA6  7|    |8   PA5 RX-
//               +----+

bool ledOn = false;
int txHighReading = 0;
int rxHighReading = 0;
int highReading = 0;
int highReadingClear = 0;
int currentlyReading = RX;

int main(void) {
	setup();
	
	while(1) {
		
		// Turn on the RX or TX
		if (currentlyReading == RX) {
			PORTA |= (1<<rxPlusControlPin);
			watchdog_sleep(T16MS);
			PORTA |= (1<<rxMinusControlPin);
			watchdog_sleep(T16MS);
		}
		else {
			PORTA |= (1<<txPlusControlPin);
			watchdog_sleep(T16MS);
			PORTA |= (1<<txMinusControlPin);
			watchdog_sleep(T16MS);
		}
		
		sbi(ADCSRA, ADEN); // Turn on ADC
		analog_read(adcPin); // First reading is ignored
		_delay_ms(1);
		int adcValue = analog_read(adcPin); // Read ADC
		cbi(ADCSRA, ADEN); // Turn off ADC
		
		// Turn off the RX or TX
		if (currentlyReading == RX) {
			PORTA &= ~(1<<rxMinusControlPin);
			watchdog_sleep(T16MS);
			PORTA &= ~(1<<rxPlusControlPin);
		}
		else {
			PORTA &= ~(1<<txMinusControlPin);
			watchdog_sleep(T16MS);
			PORTA &= ~(1<<txPlusControlPin);
		}
		
		// If close to 0 reading then the wire is disconnected
		if (adcValue < 30) {
			PORTB |= (1<<ledPin); // Maybe have it blink?
			ledOn = true;
		}
		
		// Only do something if the current and last reading is more than the disconnected wire rate
		if (adcValue > 30 && highReading > 30) {
			int difference = highReading - adcValue;
			
			// Check for high to low transition of 146mV, the difference between last reading and adcValue
			if (difference > 50) {
				PORTB |= (1<<ledPin);
				ledOn = true;
			}
			else if (ledOn == true && ((highReading - adcValue) < 17)) { // Has it risen to 50mV near the high point or exceeded it?
				PORTB &= ~(1<<ledPin);
				ledOn = false;
				highReadingClear = 0;
				highReading = 0;
			}
		}
		
		// LED off, continue normal switching between TX and RX every 4 seconds
		if (ledOn == false) {
			if (adcValue > highReading) { // Is this the new highest reading?
				highReading = adcValue;
			}
			if (highReadingClear >= 5) { // Save high reading
				highReadingClear = 0;
				
				// Switch between RX/TX
				if (currentlyReading == RX) {
					currentlyReading = TX;
					rxHighReading = highReading;
					highReading = txHighReading;
				}
				else {
					currentlyReading = RX;
					txHighReading = highReading;
					highReading = rxHighReading;
				}
			}
			highReadingClear++;
		}
		
		// Sleep for a bit
		watchdog_sleep(T1S);
	}
}

ISR(WDT_vect) { }

ISR(ADC_vect) { }
